/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <math.h>
#include "mex.h"
#include "chroma.h"
#include "resample.h"

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {

	/* Declarations */
	int n,m;
	int numsamples,numchannels,numframes;
	int downratio,winlen,hopsize,downsiglen;
	double fmin,fmax,Fs,Fsdown;
	int bpo;
	double *downsig,*out;
	chroma *ch;
	mxArray *p;
	mxArray *temp;

	/* Check number of arguments */
	if (nrhs<1) mexErrMsgTxt( "Not enough input arguments.");
	if (nlhs>1) mexErrMsgTxt( "Too many output arguments.");

	/* Input */
	numsamples = (int)mxGetDimensions(prhs[0])[0];
	numchannels = (int)mxGetDimensions(prhs[0])[1];
	if (numchannels!=1) mexErrMsgTxt( "Input signal must be one column vector.");

	/* Parameters */
	if (nrhs>=2) {
		if (!mxIsStruct(prhs[1])) mexErrMsgTxt("Parameters must be a structure.");
		p = (mxArray*)prhs[1];
	} else {
		p = mxCreateStructMatrix(0,0,0,NULL);
	}
	Fs = 44100;
	temp = mxGetField(p, 0, "fs");
	if (temp!=NULL) Fs = mxGetPr(temp)[0];
	downratio = 4;
	temp = mxGetField(p, 0, "downratio");
	if (temp!=NULL) downratio = (int)mxGetPr(temp)[0];
	Fsdown = Fs/(double)downratio;
	fmin = 73.4162;
	temp = mxGetField(p, 0, "fmin");
	if (temp!=NULL) fmin = mxGetPr(temp)[0];
	fmax = fmin * 8;
	temp = mxGetField(p, 0, "fmax");
	if (temp!=NULL) fmax = mxGetPr(temp)[0];
	hopsize = 2048;
	temp = mxGetField(p, 0, "hopsize");
	if (temp!=NULL) hopsize = (int)mxGetPr(temp)[0];
	bpo = 36;
	temp = mxGetField(p, 0, "bpo");
	if (temp!=NULL) bpo = (int)mxGetPr(temp)[0];

	/* Init chromagram */
	ch = new chroma( fmin, fmax, bpo, Fsdown );
	winlen = ch->fftlength;

	/* Downsample signal */
	downsiglen = (int)ceil((double)numsamples/(double)downratio);
	numframes = (downsiglen/hopsize)+1;
	downsiglen += 3*winlen/2;
	downsig = (double*)mxCalloc(downsiglen,sizeof(double));
	for (n=0;n<downsiglen;n++) downsig[n] = 0;
	downsample( mxGetPr(prhs[0]), numsamples, downsig+winlen/2, downratio, 1 );

	/* Init output */
	if (nlhs>0) plhs[0] = mxCreateDoubleMatrix(bpo,numframes,mxREAL);
	out = new double[bpo];

	/* Compute chromagram */
	for (n=0;n<numframes;n++) {
		ch->compute( downsig+hopsize*n, out );
		if (nlhs>0) {
			for (m=0;m<bpo;m++)
				mxGetPr(plhs[0])[n*bpo+m] = out[m];
		}
	}

	/* Clean */
	mxFree( downsig );
	delete[]( out );
	delete( ch );
}

