/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <math.h>
#include "mex.h"
#include "mfcc.h"

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {

	/* Declarations */
	double *signal,*output,*win;
	double fs,lowfreq,highfreq;
	int numsamples,numchannels;
	int winlen,hopsize,numfilters,numcoefs;
	int n,m,numframes;
	mfcc *mfccoefs;
	mxArray *p;
	mxArray *temp;
	double *buffer;
	bool setwin=false;

	/* Check number of arguments */
	if (nrhs<1) mexErrMsgTxt( "Not enough input arguments.");
	if (nlhs>1) mexErrMsgTxt( "Too many output arguments.");

	/* Input Signal */
	signal = mxGetPr(prhs[0]);
	numsamples = (int)mxGetDimensions(prhs[0])[0];
	numchannels = (int)mxGetDimensions(prhs[0])[1];
	if (numchannels!=1) mexErrMsgTxt( "Input signal must be one column vector.");

	/* Parameters */
	if (nrhs>=2) {
		if (!mxIsStruct(prhs[1])) mexErrMsgTxt("Parameters must be a structure.");
		p = (mxArray*)prhs[1];
	} else {
		p = mxCreateStructMatrix(0,0,0,NULL);
	}
	fs = 44100.0;
	temp = mxGetField(p, 0, "fs");
	if (temp!=NULL) fs = mxGetPr(temp)[0];
	winlen = 1024;
	temp = mxGetField(p, 0, "winlen");
	if (temp!=NULL) winlen = (int)mxGetPr(temp)[0];
	hopsize = 1024;
	temp = mxGetField(p, 0, "hopsize");
	if (temp!=NULL) hopsize = (int)mxGetPr(temp)[0];
	win = (double*) mxCalloc( winlen, sizeof(double) );
	for (n=0;n<winlen;n++) win[n] = 0.54 - 0.46*cos(2*M_PI*n/winlen);
	temp = mxGetField(p, 0, "win");
	if (temp!=NULL) {
		mxFree( win );
		win = mxGetPr(temp);
		setwin = true;
	}
	lowfreq = 20;
	temp = mxGetField(p, 0, "lowfreq");
	if (temp!=NULL) lowfreq = mxGetPr(temp)[0];
	highfreq = 16000;
	temp = mxGetField(p, 0, "highfreq");
	if (temp!=NULL) highfreq = mxGetPr(temp)[0];
	numfilters = 40;
	temp = mxGetField(p, 0, "numfilters");
	if (temp!=NULL) numfilters = (int)mxGetPr(temp)[0];
	numcoefs = 13;
	temp = mxGetField(p, 0, "numcoefs");
	if (temp!=NULL) numcoefs = (int)mxGetPr(temp)[0];

	/* Init detection function object */
	mfccoefs = new mfcc(fs,winlen,win,lowfreq,highfreq,numfilters,numcoefs);

	/* Init output */
	numframes = (numsamples-winlen)/hopsize+1;
	if (nlhs>0) {
		plhs[0] = mxCreateDoubleMatrix(numcoefs,numframes,mxREAL);
		output = mxGetPr( plhs[0] );
	}
	buffer = (double*) mxCalloc(numcoefs,sizeof(double));

	/* Computation loop */
	for (n=0;n<numframes;n++) {
		mfccoefs->compute( signal+hopsize*n, buffer );
		if (nlhs>0)
			for (m=0;m<numcoefs;m++)
				output[numcoefs*n+m] = buffer[m];
	}

	/* Clean */
	mxFree( buffer );
	if (setwin==false) mxFree( win );
	delete( mfccoefs );
}

