/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <math.h>

#include "resample.h"

void downsample( double *in, int inlen, double *out, int ratio, int normalize ) {

	/* Declarations */
	int n,m;
	int idx_ratio,filterlen;
	double *zi,*filter_a,*filter_b;
	double inputval,outputval;
	double eps = pow(2,-52);
	double max = 0;

	/* Find maximum */
	if (normalize==1)
		for (n=0;n<inlen;n++)
			if (in[n]>max)
				max = in[n];

	if (ratio==1) {

		/* Copy */
		for (n=0;n<inlen;n++) {
			out[n] = in[n];
			if (normalize==1) out[n] /= (max+eps);
		}

	} else {

		/* Init */
		idx_ratio = (int)round(log2((double)ratio))-1;
		filterlen = 8;
		filter_a = (double*)filters_down_a[idx_ratio];
		filter_b = (double*)filters_down_b[idx_ratio];
		zi = new double[filterlen-1];
		for (n=0;n<filterlen-1;n++) zi[n] = 0;


		/* Filter + Remove samples */
		for (n=0;n<inlen;n++) {
			inputval = in[n];
			if (normalize==1) inputval /= max+eps;
			outputval = filter_b[0]*inputval+zi[0];
			if (n%ratio==0) out[n/ratio] = outputval;
			for (m=0;m<filterlen-2;m++)
				zi[m] = filter_b[m+1]*inputval+zi[m+1]-filter_a[m+1]*outputval;
			zi[filterlen-2] = filter_b[filterlen-1]*inputval-filter_a[filterlen-1]*outputval;
		}

		/* Clean */
		delete[]( zi );
	}
}

void upsample( double *in, int inlen, double *out, int ratio ) {

	
	/* Declarations */
	int n,m;
	int idx_ratio;
	int filterlen,odlen;
	double *filter,*od,*zi;
	double inputval,outputval;

	/* Init  */
	idx_ratio = (int)round(log2((double)ratio))-1;
	filterlen = filters_up_len[idx_ratio];
	filter = (double*)filters_up_b[idx_ratio];
	odlen = filterlen-1;
	od = new double[odlen];
	zi = new double[odlen];

	/* Initial conditions */
	for (n=0;n<odlen;n++)
		od[n] = 0;
	for (n=0;n<odlen/ratio;n++)
		od[n*ratio] = 2*in[0] - in[odlen/ratio-n];
	for (n=0;n<odlen;n++)
		zi[n] = 0;
	for (n=0;n<odlen;n++) {
		for (m=0;m<filterlen-2;m++)
			zi[m] = filter[m+1]*od[n]+zi[m+1];
		zi[filterlen-2] = filter[filterlen-1]*od[n];
	}

	/* Insert zeros between samples + Filter */
	for (n=0;n<inlen*ratio;n++) {
		inputval = 0;
		if (n%ratio==0) inputval = in[n/ratio];
		outputval = filter[0]*inputval+zi[0];
		if (n>=odlen/2) out[n-odlen/2] = outputval;
		for (m=0;m<filterlen-2;m++)
			zi[m] = filter[m+1]*inputval+zi[m+1];
		zi[filterlen-2] = filter[filterlen-1]*inputval;
	}

	/* Final conditions */
	for (n=0;n<odlen;n++)
		od[n] = 0;
	for (n=0;n<odlen/ratio;n++)
		od[n*ratio] = 2*in[inlen-1] - in[inlen-2-n];
	for (n=0;n<odlen/2;n++) {
		out[n+inlen*ratio-odlen/2] = filter[0]*od[n]+zi[0];
		for (m=0;m<filterlen-2;m++)
			zi[m] = filter[m+1]*od[n]+zi[m+1];
		zi[filterlen-2] = filter[filterlen-1]*od[n];
	}
		

	/* Clean */
	delete[]( od );
	delete[]( zi );
}

